function [df, dg] = OTCCVgra(x, baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg)
%OTCCVGRA  Evaluates gradients of objective function & constraints for OPF.
%   [df, dg] = OTCCVgra(x, baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg)

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialize -----
vVg = varVg(alg);

%% constant
j = sqrt(-1);

%% define named indices into bus, gen, branch matrices
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;

%% set up indexing for x
nb = size(bus, 1);
nl = size(branch, 1);
npv	= length(pv);
npq	= length(pq);
ng = npv + 1;
j1 = 1;			j2 = npv;			%% j1:j2 - V angle of pv buses
j3 = j2 + 1;	j4 = j2 + npq;		%% j3:j4 - V angle of pq buses
if vVg
	j5 = j4 + 1;	j6 = j4 + nb;	%% j5:j6 - V mag of all buses
else
	j5 = j4 + 1;	j6 = j4 + npq;	%% j5:j6 - V mag of pq buses
end
j7 = j6 + 1;	j8 = j6 + ng;		%% j7:j8 - P of generators
j9 = j8 + 1;	j10= j8 + ng;		%% j9:j10- c of generators

%% grab Pg
Pg = x(j7:j8) * baseMVA;					%% power generation in MW
c = x(j9:j10);								%% generator costs in $/hr

%%----- evaluate partials of objective function -----
%% generator info
on = find(gen(:, GEN_STATUS));				%% which generators are on?
gbus = gen(on, GEN_BUS);					%% what buses are they at?

%% compute values of objective function partials
df = [	zeros(j8, 1);				%% partial w.r.t. Va, Vm, Pg
		ones(ng, 1)	];				%% partial w.r.t. c

%%----- evaluate partials of constraints -----
if nargout > 1
	%% reconstruct V
	if vVg
		Va = zeros(nb, 1);
		Va(ref) = angle(V(ref));
		Vm = x(j5:j6);
	else
		Va = angle(V);	Vm = abs(V);
		Vm(pq) = x(j5:j6);
	end
	Va(pv) = x(j1:j2);
	Va(pq) = x(j3:j4);
	V = Vm .* exp(j * Va);
	
	%% compute partials of injected bus powers
	[dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V);		%% w.r.t. V
	dSbus_dPg = sparse(gbus, 1:ng, -1, nb, ng);		%% w.r.t. Pg
	
	%% compute partials of line flows w.r.t. V
	[dSf_dVa, dSf_dVm, dSt_dVa, dSt_dVm, Sf, St] = dSbr_dV(branch, Yf, Yt, V);

	%% line limits are w.r.t apparent power, so compute partials of apparent power
	[dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm] = ...
						dAbr_dV(dSf_dVa, dSf_dVm, dSt_dVa, dSt_dVm, Sf, St);	

	%% cost-c w.r.t everything
	if CCV(alg) == 'Q'						%% quadratic
		dcosts = zeros(ng, 1);
		for i = 1:ng
			dcosts(i) = polyval(polyder( gencost( on(i), COST:(COST+gencost(on(i), N)-1) ) ), Pg(i)) * baseMVA;
		end
		cc = [sparse(ng,j6), spdiags(dcosts,0,ng,ng), -speye(ng,ng)];
	else	%% CCV(alg) == 'P', 			%% piece-wise linear
		nsegs = gencost(on, N) - 1;			%% number of cost constraints for each gen
		ncc = sum(nsegs);					%% total number of cost constraints
		dcosts = sparse(ncc, 2*ng);
		for i = 1:ng
			xx = gencost(on(i),		COST:2:( COST + 2*(nsegs(i))	))';
			yy = gencost(on(i),	(COST+1):2:( COST + 2*(nsegs(i)) + 1))';
			i1 = 1:nsegs(i);
			i2 = 2:(nsegs(i) + 1);
			m = (yy(i2) - yy(i1)) ./ (xx(i2) - xx(i1));
			dcosts(sum(nsegs(1:(i-1))) + [1:nsegs(i)], [i, ng+i]) = ...
								[ m * baseMVA, -1 * ones(nsegs(i), 1) ];
		end
		cc = [sparse(ncc,j6), dcosts];
	end

	%% evaluate partials of constraints
	if vVg
		dg = [
			%% equality constraints
			real(dSbus_dVa(gbus,pv)), real(dSbus_dVa(gbus,pq)), ...
						real(dSbus_dVm(gbus,:)), real(dSbus_dPg(gbus,1:ng)), ...
						sparse(ng,ng);											%% P mis for ref & pv
			real(dSbus_dVa(pq,pv)), real(dSbus_dVa(pq,pq)), ...
						real(dSbus_dVm(pq,:)), real(dSbus_dPg(pq,1:ng)), ...
						sparse(npq,ng);											%% P mis for pq
			imag(dSbus_dVa(pq,pv)), imag(dSbus_dVa(pq,pq)), ...
						imag(dSbus_dVm(pq,:)), real(dSbus_dPg(pq,1:ng)), ...
						sparse(npq,ng);											%% Q mis for pq
		
			%% inequality constraints (variable limits, voltage & real generation)
			sparse(nb, j4), -speye(nb,nb), sparse(nb,ng), sparse(nb,ng);		%% Vmin for all buses
			sparse(nb, j4),  speye(nb,nb), sparse(nb,ng), sparse(nb,ng);		%% Vmax for all buses
			sparse(ng, j6), -speye(ng,ng), sparse(ng,ng);						%% Pmin for generators
			sparse(ng, j6),  speye(ng,ng), sparse(ng,ng);						%% Pmax for generators
		
			%% inequality constraints (reactive generation & line flow limits)
			-imag(dSbus_dVa(gbus,pv)), -imag(dSbus_dVa(gbus,pq)), ...
						-imag(dSbus_dVm(gbus,:)), sparse(ng,2*ng);				%% gen minQ
			imag(dSbus_dVa(gbus,pv)), imag(dSbus_dVa(gbus,pq)), ...
						imag(dSbus_dVm(gbus,:)), sparse(ng,2*ng);				%% gen maxQ
% 			real(dSf_dVa(:,pv)), real(dSf_dVa(:,pq)), real(dSf_dVm), ...
% 						sparse(nl,2*ng);										%% Pf limit
% 			real(dSt_dVa(:,pv)), real(dSt_dVa(:,pq)), real(dSt_dVm), ...
% 						sparse(nl,2*ng);										%% Pt limit
			dAf_dVa(:,pv), dAf_dVa(:,pq), dAf_dVm, sparse(nl,2*ng);				%% |Sf| limit
			dAt_dVa(:,pv), dAt_dVa(:,pq), dAt_dVm, sparse(nl,2*ng);				%% |St| limit
			
			%% inequality constraints on generator cost
			cc;
		]';
	else
		dg = [
			%% equality constraints
			real(dSbus_dVa(gbus,pv)), real(dSbus_dVa(gbus,pq)), ...
						real(dSbus_dVm(gbus,pq)), real(dSbus_dPg(gbus,1:ng)), ...
						sparse(ng,ng);											%% P mis for ref & pv
			real(dSbus_dVa(pq,pv)), real(dSbus_dVa(pq,pq)), ...
						real(dSbus_dVm(pq,pq)), real(dSbus_dPg(pq,1:ng)), ...
						sparse(npq,ng);											%% P mis for pq
			imag(dSbus_dVa(pq,pv)), imag(dSbus_dVa(pq,pq)), ...
						imag(dSbus_dVm(pq,pq)), real(dSbus_dPg(pq,1:ng)), ...
						sparse(npq,ng);											%% Q mis for pq
		
			%% inequality constraints (variable limits, voltage & real generation)
			sparse(npq,j4), -speye(npq,npq), sparse(npq,ng), sparse(npq,ng);	%% Vmin for pq buses
			sparse(npq,j4),  speye(npq,npq), sparse(npq,ng), sparse(npq,ng);	%% Vmax for pq buses
			sparse(ng, j6), -speye(ng,ng), sparse(ng,ng);						%% Pmin for generators
			sparse(ng, j6),  speye(ng,ng), sparse(ng,ng);						%% Pmax for generators
		
			%% inequality constraints (reactive generation & line flow limits)
			-imag(dSbus_dVa(gbus,pv)), -imag(dSbus_dVa(gbus,pq)), ...
						-imag(dSbus_dVm(gbus,pq)), sparse(ng,2*ng);				%% gen minQ
			imag(dSbus_dVa(gbus,pv)), imag(dSbus_dVa(gbus,pq)), ...
						imag(dSbus_dVm(gbus,pq)), sparse(ng,2*ng);				%% gen maxQ
% 			real(dSf_dVa(:,pv)), real(dSf_dVa(:,pq)), real(dSf_dVm(:,pq)), ...
% 						sparse(nl,2*ng);										%% Pf limit
% 			real(dSt_dVa(:,pv)), real(dSt_dVa(:,pq)), real(dSt_dVm(:,pq)), ...
% 						sparse(nl,2*ng);										%% Pt limit
			dAf_dVa(:,pv), dAf_dVa(:,pq), dAf_dVm(:,pq), sparse(nl,2*ng);		%% |Sf| limit
			dAt_dVa(:,pv), dAt_dVa(:,pq), dAt_dVm(:,pq), sparse(nl,2*ng);		%% |St| limit
			
			%% inequality constraints on generator cost
			cc;
		]';
	end
	
	%% make full so optimization toolbox doesn't go wacky
	dg = full(dg);
end

return;
