function runpf(casename, verbose, pretty, ugly)
%RUNPF  Runs a power flow.
%
%   runpf(casename, verbose, pretty, ugly)
%
%   Runs a full Newton-Raphson power flow where casename is the name of
%   the m-file (without the .m extension) containing the power flow data. If
%   verbose is true it prints some extra things while it's solving, and if
%   pretty is true it pretty prints the output. If ugly is true it prints
%   raw data for our Perl database interface. Default parameters are
%   'case', 1, 1 and 0.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% define named indices into bus, gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;

%% default arguments
if nargin < 4
	ugly = 0;						%% don't print the raw stuff
	if nargin < 3
		pretty = 1;					%% print the pretty stuff
		if nargin < 2
			verbose = 1;			%% be verbose by default
			if nargin < 1
				casename = 'case';	%% default data file is 'case.m'
			end
		end
	end
end

%% read data & convert to internal bus numbering
[baseMVA, bus, gen, branch] = feval(casename);
[i2e, bus, gen, branch] = ext2int(bus, gen, branch);

%% get bus index lists of each type of bus
[ref, pv, pq] = bustypes(bus, gen);

%% build admittance matrices
[Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);

%% compute complex bus power injections (generation - load)
Sbus = makeSbus(baseMVA, bus, gen);

%% initialize complex voltage vector from data from case file
V0	= bus(:, VM) .* exp(sqrt(-1) * pi/180 * bus(:, VA));
V0(gen(:, GEN_BUS)) = gen(:, VG) ./ abs(V0(gen(:, GEN_BUS))) .* V0(gen(:, GEN_BUS));

%% run the power flow
t0 = clock;
[V, success, iterations] = newtonpf(Ybus, Sbus, V0, ref, pv, pq, 1e-8, 10, verbose);

%% compute flows etc.
[bus, gen, branch] = pfsoln(baseMVA, bus, gen, branch, Ybus, Yf, Yt, V, ref, pv, pq);
et = etime(clock, t0);

%% convert back to original bus numbering & print results
[bus, gen, branch] = int2ext(i2e, bus, gen, branch);
printpf(bus, gen, branch, success, et, 1, pretty, ugly);

return;
